@@ -1,33 +1,98 @@
 # constants.py
+import re
 from collections.abc import Mapping
 from collections import OrderedDict
 import datetime
-# from decimal import Decimal
 import logging
 import numpy as np
 import pandas as pd
 from pathlib import Path
 from pytz import timezone
+import shutil
 import string
+from urllib.request import urlretrieve
+
 
 log = logging.getLogger(__name__)
-__version__ = '0.0.9rc'
+
+
+PACKAGE_DIR = Path(__file__).absolute().resolve().parent
+SRC_DIR = PACKAGE_DIR.parent
+REPO_DIR = SRC_DIR.parent
+
+
+def get_version():
+    """ Look within setup.cfg for version = ... and within setup.py for __version__ = """
+
+    # setup.cfg
+    with (REPO_DIR / 'setup.cfg').open() as fin:
+        for line in fin:
+            matched = re.match(r'\s*version\s*=\s*([.0-9abrc])\b', line)
+            if matched:
+                return (matched.groups()[-1] or '').strip()
+
+    # setup.py
+    try:
+        version = next(iter(
+            line for line in (REPO_DIR / 'setup.py').open() if line.startswith('__version__ = ')))
+        return version[len('__version__ = '):].strip('"').strip("'")
+    except Exception as e:
+        print('ERROR: Unable to find version in setup.py.')
+        print(e)
+
+
+__version__ = get_version()
+
+
 log.debug(f'Running {__name__} version {__version__} ...')
-LOGLEVEL = logging.ERROR
+# LOGLEVEL = logging.ERROR
 
-data_dir_names_ = ['.nlpia2-data', 'nessvec-data']
-DATA_DIR = Path('~').expanduser().resolve().absolute() / data_dir_names_[0]
 
-for dir_name in data_dir_names_:
-    DATA_DIR = DATA_DIR.parent / dir_name
+############################################################################
+# setup DATA_DIR
+# TODO: setup.py should handle all this
+
+
+__version__ = next(iter(
+    line for line in (REPO_DIR / 'setup.py').open() if line.startswith('__version__ = ')))
+__version__ = __version__[len('__version__ = '):].strip('"').strip("'")
+HOME_DIR = Path.home()
+SHARED_DATA_DIR_NAMES = ['.nlpia2-data', 'nessvec-data']
+DATA_DIR_NAME = SHARED_DATA_DIR_NAMES[-1]
+DATA_DIR = HOME_DIR / DATA_DIR_NAME
+for DATA_DIR_NAME in SHARED_DATA_DIR_NAMES:
+    DATA_DIR = HOME_DIR / DATA_DIR_NAME
     if DATA_DIR.is_dir():
         break
+DATA_CONSTANTS_BASEURL = 'https://gitlab.com/tangibleai/nessvec/-/raw/main/src/nessvec/data/constants'
+DATA_CONSTANTS_FILENAMES = [
+    'LANG_INFO.csv',
+    'LANG_ISO.csv',
+    'tlds-from-iana.csv',
+    'uri-schemes.xhtml.csv',
+]
+DATA_CONSTANTS_URLS = [
+    f'{DATA_CONSTANTS_BASEURL}/{fn}' for fn in DATA_CONSTANTS_FILENAMES
+]
 
-# the last of the data_dir possibilities is the default
-if not DATA_DIR.is_dir():
-    DATA_DIR.mkdir()
-log.debug(f'Storing vectors, models, and benchmark datasets in DATA_DIR={DATA_DIR}')
+if not (DATA_DIR / 'constants' / 'LANG_ISO.csv').is_file():
+    (DATA_DIR / 'constants').mkdir(parents=True, exist_ok=True)
+    try:
+        shutil.copytree(str(PACKAGE_DIR / 'data'), str(DATA_DIR), dirs_exist_ok=True)
+    except FileNotFoundError:
+        for fn in DATA_CONSTANTS_FILENAMES:
+            url = f'{DATA_CONSTANTS_BASEURL}/{fn}'
+            fp = DATA_DIR / 'constants' / fn
+            urlretrieve(url, str(fp), show_progress=False, exist_ok=True)
 
+        # the last of the data_dir possibilities is the default
+        # copy data files from python package to USER_DATA_DIR
+
+log.debug(f'Storing vectors, models, and benchmark datasets in USER_DATA_DIR={DATA_DIR}')
+
+# TODO: setup.py should handle all this
+# setup DATA_DIR
+#############################################################################
 
 ANALOGY_URLS = [
     # SAT(acronym for Scholastic Aptitude Test), 5 610 questions divided into 374 semantic classes.
@@ -76,9 +141,81 @@ LARGE_FILES = dict(
     for (u, fn) in zip(ANALOGY_URLS, ANALOGY_FILEPATHS)
 )
 
+#######################################################
+# for nessvec.translation
+
+""" ISO 2-letter abbreviations
+
+>>> import pandas as pd
+>>> dfs = pd.read_html('https://en.wikipedia.org/wiki/List_of_ISO_639-1_codes')
+>>> dfs[1]
+    ISO language name  ...                                              Notes
+0           Abkhazian  ...                               also known as Abkhaz
+1                Afar  ...                                                NaN
+2           Afrikaans  ...                                                NaN
+3                Akan  ...         macrolanguage, Twi is tw/twi, Fanti is fat
+4            Albanian  ...  macrolanguage, called "Albanian Phylozone" in ...
+..                ...  ...                                                ...
+178             Xhosa  ...                                                NaN
+179           Yiddish  ...  macrolanguage. Changed in 1989 from original I...
+180            Yoruba  ...                                                NaN
+181    Zhuang, Chuang  ...                                      macrolanguage
+182              Zulu  ...                                                NaN
+
+[183 rows x 6 columns]
+>>> dfs[1].columns
+Index(['ISO language name', '639-1', '639-2/T', '639-2/B', '639-3', 'Notes'], dtype='object')
+>>> df = dfs[1]
+>>> df[df.columns[:2]]
+    ISO language name 639-1
+0           Abkhazian    ab
+1                Afar    aa
+2           Afrikaans    af
+3                Akan    ak
+4            Albanian    sq
+..                ...   ...
+178             Xhosa    xh
+179           Yiddish    yi
+180            Yoruba    yo
+181    Zhuang, Chuang    za
+182              Zulu    zu
+
+[183 rows x 2 columns]
+>>> name2abbrev = list(zip(*df[df.columns[:2]].values.T))
+>>> name2abbrev = dict([(n.lower()[:2], a) for (n, a) in name2abbrev])
+>>> len(name2abbrev)
+104
+>>> name2abbrev['po']  # Portuguese
+'pt'
+>>> abbrev = df[df.columns[1]].values
+>>> name2abbrev.update(dict(zip(*[abbrev, abbrev])))
+>>> name2abbrev = pd.Series(name2abbrev)
+>>> name2abbrev
+ab    ab
+af    af
+ak    ak
+al    sq
+am    am
+      ..
+ty    ty
+ug    ug
+cy    cy
+fy    fy
+za    za
+Length: 205, dtype: object
+
+>>> name2abbrev.name = 'ISO'
+>>> name2abbrev.to_csv('LANG_ISO.csv')
+>>> n2a = pd.read_csv('LANG_ISO.csv', index_col=0)
+"""
+
+LANG_ISO = pd.read_csv(DATA_DIR / 'constants' / 'LANG_ISO.csv', index_col=0)
+LANG_ISO = LANG_ISO[LANG_ISO.columns[0]].to_dict()
+LANG_INFO = pd.read_csv(DATA_DIR / 'constants' / 'LANG_INFO.csv', index_col=0)
+
 
 #########################################################
-# qary.constants
+# from qary.constants
 
 LOGGING_FORMAT = '%(asctime)s.%(msecs)d %(levelname)-4s %(filename)s:%(lineno)d %(message)s'
 LOGGING_DATEFMT = '%Y-%m-%d:%H:%M:%S'
@@ -96,9 +233,9 @@ DEFAULT_TZ = timezone('UTC')
 MAX_LEN_FILEPATH = 1023  # on OSX `open(fn)` raises OSError('Filename too long') if len(fn)>=1024
 
 #####################################################################################
-# pugnlp.constants
+# from pugnlp.constants
 
-tld_iana = pd.read_csv(Path(DATA_DIR) / 'constants' / 'tlds-from-iana.csv', encoding='utf8')
+tld_iana = pd.read_csv(DATA_DIR / 'constants' / 'tlds-from-iana.csv', encoding='utf8')
 tld_iana = OrderedDict(sorted(zip((tld.strip().lstrip('.') for tld in tld_iana.domain),
                                   [(sponsor.strip(), -1) for sponsor in tld_iana.sponsor]),
                               key=lambda x: len(x[0]),
