@@ -232,7 +232,7 @@ def download(url, data_dir=DATA_DIR, filepath=None, filename=None, show_progress
 
     if not filepath:
         if not filename:
-            filename = os.path.split(url)[-1]  # this separates url by '/' and takes the last part
+            filename = str(url).split('/')[-1]
         filepath = data_dir / filename
     filepath = ensure_path(filepath)
 
